#++ ---------------- sympower.mu ---------------------

Author: Winfried Fakler,  University of Karlsruhe
Date Created: 24 April  1996
Last Update : 12 August 1996

Description:
symmetricPower( L, y(x), m ) computes the m-th symmetricPower of the 
linear homogeneous differential equation L. This is the differential 
equation of lowest order whose solution space consists exactly of all 
possible m-th powerproducts of solutions of L.

See: Singer, M. F. (1985). Solving homogeneous linear differential 
     equations in terms of second order linear differential equations. 
     Amer. J. Math. 107

Example:

>> eq := diff(y(x),x,x)+1/x*diff(y(x),x)-1/(4*x^2)*y(x):
>> ode::symmetricPower(eq, y(x), 2);
 
                 3 diff(y(x), x, x)
                 ------------------ + diff(y(x), x, x, x)
                         x

Hurwitz-equation:
>> eq:=x^2*(x-1)^2*diff(y(x),x,x,x)+(7*x-4)*x*(x-1)*diff(y(x),x,x)+
 (72/7*(x^2-x)-20/9*(x-1)+3/4*x)*diff(y(x),x)+(72*11/7^3*(x-1)+5/8+2/63)*y(x):
>> ode::symmetricPower(eq, y(x), 2);
------------------------------------------------- ++#

#-- 
symmetricPower(eq, z, m):    (arguments are tested additionally)
 Input : eq - a homogeneous linear ordinary differential equation
         z  - the operator of eq (e.g. something of the form y(x) )
         m  - a positive Integer
 Output: the m-th symmetric power of eq
--#  


ode::symmetricPower := proc(eq, z, m) 
  begin
    eq := ode::isLODE(eq, z, "Lode");
    case nops(eq) 
      of 1 do return(error("not an ordinary linear differential equation ". 
                            "over the rational functions"));
      of 4 do if eq[4] < 1 then
                 return(error("only defined for positive order equations"))
              end_if;
      otherwise
        if domtype(m) <> DOM_INT or m < 1
          then return(error("only defined for positive integers"))
        end_if
    end_case;
    ode::symPower(eq, m)
  end_proc:


#-- ------------------------------------------------
symPower(eq, z, x, n, m):  
 Input : eq - a homogeneous linear ordinary differential equation
         z  - name of the operator of eq 
         x  - the dependant variable of the operator z
         n  - order of eq
         m  - a positive Integer
 Output: the m-th symmetric power of eq
--#  


ode::symPower := proc(eq, z, x, n, m) 
  local t0, c, Y, i, j, peq, 
        ansatz, dimBasis, newDimBasis, diffvars, solset, term;
  begin
    t0 := time();
    z := z(x);
    Y := z^m;
    peq := poly(eq, [diff(z, x$n)]);
    term := expand(- tcoeff(peq)/lcoeff(peq));
    ansatz := c[0]*Y;
    diffvars := [diff(z, x$j) $ j=0..n-1];
    for i from 1 to n-1 do
      Y := diff(Y, x);
      ansatz := ansatz + c[i]*Y;
    end_for;    
    i := n-1;
    newDimBasis := nterms(ansatz, diffvars);
    repeat
      i := i+1;
      dimBasis := newDimBasis;
      userinfo(5, "order is now: ", i);
      Y := expand(subs(diff(Y, x), diff(z, x$n)=term));
      ansatz := expand(ansatz + c[i]*Y);
      newDimBasis := nterms(ansatz, diffvars);
    until dimBasis = newDimBasis end_repeat; 
    repeat
      userinfo(5, "...solve ansatz");
      [coeff(poly(subs(ansatz, c[i]=1), diffvars))];
      solset := linsolve({op(%, j)=0 $j=1..dimBasis}, {c[j] $j=0..i-1});
      if solset <> FAIL and solset <> null() then
        userinfo(10, "Time: ", time()-t0, "msec");
        [op({c[i]=1} union solset, j) $j=1..i+1];
        return( subs( _plus(c[j]*diff(z, x$j) $j=0..i), %) ) 
      end_if;
      i := i+1;
      userinfo(5, "order is now: ", i);
      Y := expand(subs(diff(Y, x), diff(z, x$n)=term));
      ansatz :=  expand(ansatz + c[i]*Y) 
    until FALSE end_repeat;
  end_proc: 
